/*************************************************************************
 *
 * PROJECT:  CSE477Lab3
 * FILE:     Starter.c
 * AUTHOR:   Mike Esler, 1/19/99
 *
 * DECLARER: Starter
 *
 * DESCRIPTION:
 *	  
 *
 **********************************************************************/
#include <Pilot.h>
#include <SysEvtMgr.h>
#include <SerialMgr.h>
#include "StarterRsc.h"



/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	Byte replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	Byte replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;



/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define FALSE              0
#define TRUE               1

#define SerRcvQueueSize    100
#define BaudRate           9600
#define FramingByte        'Z'

#define appVersionNum      0x01
#define appPrefID          0x00
#define appPrefVersionNum  0x01

#define ourMinVersion      sysMakeROMVersion(2,0,0,sysROMStageRelease,0)



/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
static Char  	serRcvQueue[SerRcvQueueSize];  //serial receive queue
static UInt		serRefNum;
static Boolean	serLineMode;
static Boolean	serLineStatus;
static Boolean	gotFramingByte;
static Long		numBytesRcvd;



/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
static void EVBconnect();
static void EVBdisconnect();
static void EVBupdate();



/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 ***********************************************************************/
static Err RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
	DWord romVersion;

	/*
	** see if we're on in minimum required version of the ROM or later
	*/
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
			FrmAlert (RomIncompatibleAlert);
		
			/*
			** Pilot 1.0 will continuously relaunch this app unless we
			** switch to another safe one
			*/
			if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		
		return (sysErrRomIncompatible);
		}

	return 0;
}



/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    VoidPtr
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Word objectID)
{
	FormPtr frmP;


	frmP = FrmGetActiveForm();
	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}



/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
}



/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(Word command)
{
	FormPtr frmP;
	Boolean handled = false;


	switch (command) {
	case MainOptionsAboutCSE477Lab3:
		MenuEraseStatus(0);
		frmP = FrmInitForm(InfoForm);
		FrmDoDialog(frmP);
		FrmDeleteForm(frmP);
		handled = true;
		break;
	case MainOptionsConnect:
		MenuEraseStatus(0);
		EVBconnect();
		handled = true;
		break;
	case MainOptionsDisconnect:
		MenuEraseStatus(0);
		EVBdisconnect();
		handled = true;
		break;
	case MainOptionsUpdate:
		MenuEraseStatus(0);
		EVBupdate();
		handled = true;
		break;
	}
	return handled;
}



/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;


	switch (eventP->eType) {
	case menuEvent:
		return MainFormDoCommand(eventP->data.menu.itemID);
		break;
	case frmOpenEvent:
		frmP = FrmGetActiveForm();
		MainFormInit(frmP);
		FrmDrawForm(frmP);
		handled = true;
		break;
	default:
		break;
	}
	
	return handled;
}



/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 ***********************************************************************/
static Boolean AppHandleEvent( EventPtr eventP)
{
	Word formId;
	FormPtr frmP;


	if (eventP->eType == frmLoadEvent) {
		/*
		** load the form resource
		*/
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		/*
		** Set the event handler for the form. The handler of the
		** currently active form is called by FrmHandleEvent each
		** time is receives an event.
		*/
		switch (formId) {
		case MainForm:
			FrmSetEventHandler(frmP, MainFormHandleEvent);
			break;
		default:
			/* ErrFatalDisplay("Invalid Form Load Event"); */
			break;
		}

		return true;
	}
	
	return false;
}



/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Word error;
	EventType event;


	do {
		EvtGetEvent(&event, evtWaitForever);
		
		
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);
		
		/*
		** check the serial port for data.
		*/			
		EVBupdate();

	} while (event.eType != appStopEvent);
}



/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Startup code.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 ***********************************************************************/
static Err AppStart(void)
{
	serRefNum = 0;	
	
	return 0;
}



/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Cleanup code.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void AppStop(void)
{                       
	if (serRefNum != 0) {
		SerClose(serRefNum);
		serRefNum = 0;
	}
}



/***********************************************************************
 *
 * FUNCTION:    StarterPilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 ***********************************************************************/
static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err error;


	error = RomVersionCompatible(ourMinVersion, launchFlags);
	if (error)
		return error;


	switch (cmd) {
	case sysAppLaunchCmdNormalLaunch:
		error = AppStart();
		if (error) 
			return error;
				
		FrmGotoForm(MainForm);
		AppEventLoop();
		AppStop();
		break;
	default:
		break;
	}
	
	return 0;
}



/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 ***********************************************************************/
DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}



/***********************************************************************
 *
 * FUNCTION:    EVBconnect
 *
 * DESCRIPTION: Establishes serial connection
 *
 * PARAMETERS:  
 *
 * RETURNED:
 *
 ***********************************************************************/
static void EVBconnect()
{
	int				i;
	Err				err = 0;
	SerSettingsType	settings;
	FormPtr frmP;


	/*
	** already connected?
	*/	
	if (serRefNum != 0) {
		frmP = FrmInitForm(ConnectForm);
		FrmDoDialog(frmP);
		FrmDeleteForm(frmP);
		return;
	}

			
	/*
	** initialize variables
	*/
	gotFramingByte = FALSE;
	numBytesRcvd = 0;
	
	for (i=0; i < SerRcvQueueSize; i++)
		serRcvQueue[i] = '\0';
	
	err = SysLibFind("Serial Library", &serRefNum);
	ErrFatalDisplayIf(err, "Can't find Serial Library!");
	if (err) SerClose(serRefNum);
	
	
	/*
	** open the serial connection at the specified baud rate
	*/
	err = SerOpen(serRefNum, 0, BaudRate);
	ErrFatalDisplayIf(err, "Problem opening the serial port!");
	if (err) {
		SerClose(serRefNum);
		serRefNum = 0;
	}
		
	settings.baudRate = BaudRate;
	settings.ctsTimeout = 0;
	settings.flags = serSettingsFlagStopBits1 |
	                 serSettingsFlagBitsPerChar8 |
	                 serSettingsFlagRTSAutoM;
	SerSetSettings(serRefNum, &settings);
}



/***********************************************************************
 *
 * FUNCTION:    EVBdisconnect
 *
 * DESCRIPTION: Closes serial connection
 *
 * PARAMETERS:  
 *
 * RETURNED:
 *
 ***********************************************************************/
static void EVBdisconnect()
{
	FormPtr frmP;
	

	/*
	** connected?
	*/
	if (serRefNum != 0) {
		SerClose(serRefNum);
		serRefNum = 0;
	} else {
		frmP = FrmInitForm(DisconnectForm);
		FrmDoDialog(frmP);
		FrmDeleteForm(frmP);
	}
}



/***********************************************************************
 *
 * FUNCTION:    EVBupdate
 *
 * DESCRIPTION: Receives data from the EVB and updates the screen
 *
 * PARAMETERS:  
 *
 * RETURNED:
 *
 ***********************************************************************/
static void EVBupdate()
{	
	const int xCenter = 79;
	const int yCenter = 89;

	Err err = 0;
		

	/*
	** if connected, receive any data alrady on the line
	*/	
	if (serRefNum == 0)
		return;
	
	SerReceive(serRefNum, serRcvQueue, 1, 0, &err);
	
	if (err == serErrTimeOut)
		return;
	
	
	/*
	** process the framing byte
	*/
	if ((gotFramingByte == FALSE) && (serRcvQueue[0] == FramingByte)) {
		gotFramingByte = TRUE;
		return;
	}

	
	/*
	** process the data values
	*/
	if (gotFramingByte == TRUE) {
		WinDrawInvertedChars( serRcvQueue, 1, xCenter, yCenter );
		gotFramingByte = FALSE;
	}
}